
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.030.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "ExtUtils::MakeMaker" => 0,
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "Gettext Support For the Template Toolkit Version 2",
  "dist_author" => [
    "Guido Flohr <guido.flohr\@cantanea.com>"
  ],
  "dist_name" => "Template-Plugin-Gettext",
  "dist_version" => "1.0",
  "license" => "lgpl",
  "module_name" => "Template::Plugin::Gettext",
  "recursive_test_files" => 1,
  "requires" => {
    "Cwd" => 0,
    "Encode" => 0,
    "Getopt::Long" => 0,
    "Locale::Messages" => 0,
    "Locale::TextDomain" => "1.30",
    "Locale::Util" => 0,
    "Locale::XGettext" => "0.7",
    "Template" => 0,
    "Template::Parser" => 0,
    "Template::Plugin" => 0,
    "base" => 0,
    "strict" => 0
  },
  "script_files" => [
    "bin/xgettext-tt2"
  ],
  "test_requires" => {
    "File::Spec" => 0,
    "Locale::XGettext::Util::Keyword" => 0,
    "Test::More" => 0,
    "vars" => 0
  }
);


my %fallback_build_requires = (
  "File::Spec" => 0,
  "Locale::XGettext::Util::Keyword" => 0,
  "Module::Build" => "0.28",
  "Test::More" => 0,
  "vars" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
